##############################################################
#
#                  S O L O M O N      R   C O D E
#
#                     AUTHOR:   URBANO LORENZO-SEVA
#                        URV, TARRAGONA (SPAIN)
#                            DATE: 17/06/2021
#
# DATA EXAMPLE: 20 ITEMS AND N = 1,000
#
# DATA TEXT FILE USED:    solomon_data_example.dat
# OUTPUT TEXT FILE:       solomon_subsamples.dat
#
# EXECUTE THE CODE AS:    source("solomon.r")
#
##############################################################

##############################################################
# UPDATE HERE THE NAME OF YOUR INPUT DATA FILE
# A TEXT FILE WITH NO LABELS IS EXPECTED
#
filein <- "solomon_data_example.dat"
#
# UPDATE HERE THE NAME OF YOUR OUTPUT DATA FILE
# THE FIRST COLUMN CONTAINS A VARIABLE TO POINT OUT THE
# CORRESPONDING SOLOMON SUBSAMPLE 
#
#     value 1: First subsample
#     value 2: Second subsample
#
fileout <- "solomon_subsamples.dat"
# 
##############################################################

####################  SOLOMON CODE ###########################
X <- read.table(filein)
N <- nrow(X)
m <- ncol(X)
mc <- colMeans(X)
sx <- sqrt(colSums(X^2)/N - mc^2)
ones <- as.matrix(numeric(N)+1)
Z <- as.matrix((X - ones %*% mc)/(ones %*% sx))
R <- cor(X)
eigen <- eigen(R)
ld <-1
for (i in 1:m){
   if (eigen$values[i] > 1) {
   ld <- ld +1
   }
}
s <- svd(R)
Kq <- as.matrix(s$v[1:m,1:ld])
Dq <- as.matrix(diag(s$d[1:ld]))
B <- Kq %*% solve(Dq)^(0.5)
A <- t(solve(t(B)%*%R%*%B) %*% t(B) %*% R )
A <- A * as.matrix(numeric(m)+1)%*%sign(colSums(A))
pc_ld <- Z %*% A 
w <- eigen$values/m
w_ld <- w[1:ld];
s <- pc_ld %*% w_ld
ss <- sort(s,method = "quick", index.return = TRUE) 
sss <- numeric(N) + 2
pos <- seq(1,N,2)
sss[pos] <- 1
Xs <- X[ss$ix,]
ids <- ss$ix
SX <- t(rbind(t(sss),t(Xs)))
colnames(SX)[1] <- c("SG")
sid <- sort(ids,method = "quick", index.return = TRUE) 
solomon <- SX[sid$ix,]
write.table(solomon, file=fileout, row.names=FALSE, col.names=FALSE)
####################  END OF SOLOMON CODE ######################

print("##############################################################")
print("#                                                             ")
print("#             S O L O M O N      R   C O D E                  ")
print("#                                                             ")
print("#                AUTHOR:   URBANO LORENZO-SEVA                ")
print("#                    URV, TARRAGONA (SPAIN)                   ")
print("#                                                             ")
print("#                       DATE: 17/06/2021                      ")
print("#                                                             ")
print("##############################################################")
print("#                                                             ")
print(paste("# Filein : ",filein,"    "))
print(paste("# Fileout: ",fileout,"   "))
print("#                                                             ")
